// TrainSchedDlg.cpp : implementation file
//

#include "stdafx.h"
#include "TrainSched.h"
#include "TrainSchedDlg.h"
#include "AboutDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTrainSchedDlg dialog

CTrainSchedDlg::CTrainSchedDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CTrainSchedDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CTrainSchedDlg)
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

    // Parse executable filename to generate app's INI filename.
    DWORD dwFileNameLength = GetModuleFileName( NULL, m_INIfilename.GetBuffer( MAX_PATH ), MAX_PATH );
    m_INIfilename.ReleaseBuffer( dwFileNameLength );
    // Strip of 'e' and 'x' (and CAPS versions) from end. This will remove 'exe' or 'EXE'.
    m_INIfilename.TrimRight( _T( "EX" ) );
    m_INIfilename.TrimRight( _T( "ex" ) );
    // Append "ini".
    m_INIfilename += _T( "ini" );

    // Ensure everything is initialized to valid values that are referenced later.
    m_bFirstPaint = TRUE;    // Set to FALSE in first OnPaint() call.
    m_bUse3Cols = FALSE;
    m_bTableInitialized = FALSE;

    // Empty arrays to ensure no random data present.
    m_saAbbreviations.RemoveAll();
    m_paTimes.RemoveAll();
    m_uaIndexArray.RemoveAll();
    uaIndexArray.RemoveAll();

    CWinApp* pApp = AfxGetApp();
    m_dbFileName = pApp->GetProfileString( INI_SECTION, INI_DBFILENAME, NULL );
    m_uFromStation = pApp->GetProfileInt( INI_SECTION, INI_FROM, 0 );
    m_uToStation = pApp->GetProfileInt( INI_SECTION, INI_TO, 0 );
    m_uDay = pApp->GetProfileInt( INI_SECTION, INI_DAY, 0 );

    // Read abbreviations from INI file.
    CString csAbbrevKey, csAbbrevValue;
    INT nKey = 0;

    // First element is blank.
    m_saAbbreviations.Add( _T( "" ) );
    do
    {
        nKey++;
        csAbbrevKey.Format( _T( "%s%d" ), INI_ABBREV_BASE, nKey );
        csAbbrevValue = pApp->GetProfileString( INI_ABBREV_SECTION, csAbbrevKey, INI_NO_VALUE );

        // If value exists then add it to the array.
        if ( csAbbrevValue.Compare( INI_NO_VALUE ) != 0 )
            m_saAbbreviations.Add( csAbbrevValue );
        else
            nKey = -1;
    } while ( nKey > 0 );

    // Retrieve the explanation of the abbreviations.
    csAbbrevMessage = pApp->GetProfileString( INI_ABBREV_SECTION, INI_ABBREV_MESSAGE, INI_NO_VALUE );

    // Change pipe char to carriage return or load a string saying there is no explanation.
    if ( csAbbrevMessage.Compare( INI_NO_VALUE ) != 0 )
        csAbbrevMessage.Replace( _T( "|" ), _T( "\n" ) );
    else
        csAbbrevMessage.LoadString( IDS_ABBREV_NO_MSG );

    // TODO: Hard code Irish Rail abbreviations. These will only be used if
    // the timetable file is 3 column format.
    if ( ! m_saAbbreviations.GetUpperBound() )
    {
        csAbbrevMessage = _T( "EWD: Mon to Sat|M-F: Monday to Friday|T-F: Tuesday to Friday|MO/WO/FO/SO: Mon/Wed/Fri/Sat only|WX/FX: Except Wed/Fri|FSX: Fri/Sat excluded|FSO: Fri/Sat only|M-TH: Mon to Thurs" );
        m_saAbbreviations.Add( _T( "EWD" ) );
        m_saAbbreviations.Add( _T( "M-F" ) );
        m_saAbbreviations.Add( _T( "FO" ) );
        m_saAbbreviations.Add( _T( "SO" ) );
        m_saAbbreviations.Add( _T( "T-F" ) );
        m_saAbbreviations.Add( _T( "MO" ) );
        m_saAbbreviations.Add( _T( "WO" ) );
        m_saAbbreviations.Add( _T( "WX" ) );
        m_saAbbreviations.Add( _T( "FX" ) );
        m_saAbbreviations.Add( _T( "FSX" ) );
        m_saAbbreviations.Add( _T( "FSO" ) );
        m_saAbbreviations.Add( _T( "M-TH" ) );
    }
}

void CTrainSchedDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTrainSchedDlg)
	DDX_Control(pDX, IDC_TO_COMBO, m_ToCombo);
	DDX_Control(pDX, IDC_FROM_COMBO, m_FromCombo);
	DDX_Control(pDX, IDC_DAY_COMBO, m_DayCombo);
	DDX_Control(pDX, IDC_TIME_LIST, m_Timetable);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CTrainSchedDlg, CDialog)
	//{{AFX_MSG_MAP(CTrainSchedDlg)
	ON_CBN_SELCHANGE(IDC_DAY_COMBO, OnSelchangeDayCombo)
	ON_CBN_SELCHANGE(IDC_FROM_COMBO, OnSelchangeFromCombo)
	ON_CBN_SELCHANGE(IDC_TO_COMBO, OnSelchangeToCombo)
	ON_BN_CLICKED(IDC_REVERSE_BUTTON, OnReverseButton)
	ON_WM_LBUTTONDOWN()
	ON_COMMAND(IDS_ABBREVIATIONS, OnAbbreviations)
	ON_COMMAND(IDS_ABOUTBOX, OnAboutbox)
	ON_COMMAND(IDS_SELECT_TIMETABLE, OnSelectTimetable)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTrainSchedDlg message handlers

BOOL CTrainSchedDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	CenterWindow(GetDesktopWindow());	// center to the hpc screen

	return TRUE;  // return TRUE  unless you set the focus to a control
}



void CTrainSchedDlg::OnSelchangeDayCombo() 
{
    m_uDay = m_DayCombo.GetCurSel();

    ReloadTimetable();
}

void CTrainSchedDlg::OnSelchangeFromCombo() 
{
    m_uFromStation = m_FromCombo.GetCurSel();

    ReloadTimetable();
}

void CTrainSchedDlg::OnSelchangeToCombo() 
{
    m_uToStation = m_ToCombo.GetCurSel();

    ReloadTimetable();
}

void CTrainSchedDlg::OnReverseButton() 
{
    // Reverse the combo box selections before reloading the timetable.
    int nSel = m_FromCombo.GetCurSel();
    m_FromCombo.SetCurSel( m_ToCombo.GetCurSel() );
    m_ToCombo.SetCurSel( nSel );

    m_uFromStation = m_FromCombo.GetCurSel();
    m_uToStation = m_ToCombo.GetCurSel();

    ReloadTimetable();
}

void CTrainSchedDlg::ReloadTimetable()
{
    // Delete all of the items from the list view control.
    m_Timetable.DeleteAllItems();

    // Don't display anything if the same station is in both combo boxes.
    if ( m_uFromStation == m_uToStation )
        return;

    UINT uDirection = 0;
    UINT uFromIndex = m_uFromStation;
    UINT uToIndex = m_uToStation;

    if ( uFromIndex > uToIndex )
    {
        uDirection = ( m_uaIndexArray.GetUpperBound() + 1 ) / 2;
        // Text file lists reverse direction time columns in
        // reverse order so must reference differently.
        uFromIndex = m_FromCombo.GetCount() - m_uFromStation - 1;
        uToIndex = m_ToCombo.GetCount() - m_uToStation - 1;
    }
    CPtrArray *pa = (CPtrArray *) m_paTimes.GetAt( m_uDay + uDirection );

    CString csFromTime, csToTime, csAbbrev;
    UINT uRow = 0;
    LVITEM lvi;
    lvi.mask = LVIF_TEXT;

    INT nCol0 = 0, nCol1 = 0, nCol2 = 0, nWidth;
    for ( INT nTrainNumber = 0; nTrainNumber <= pa->GetUpperBound(); nTrainNumber++ )
    {
        CStringArray *sa = (CStringArray *) pa->GetAt( nTrainNumber );

        csFromTime = sa->GetAt( uFromIndex );
        csToTime = sa->GetAt( uToIndex );

        // Only add if train stops at both stations. '-' in string if not stopping.
        if ( ( csFromTime.Find( _T( '-' ) ) == -1 ) &&
           ( csToTime.Find( _T( '-' ) ) == -1 ) )
        {
            lvi.iItem = uRow;    // Row.

            // Add the From time.
            lvi.iSubItem = 0;  // Column
            lvi.pszText = _T( "" );
            lvi.cchTextMax = sizeof( _T( "" ) );
            m_Timetable.InsertItem( &lvi );
            FormatTime( csFromTime );
            // Get width of this string.
            nWidth = m_Timetable.GetStringWidth( csFromTime );
            if ( nWidth > nCol0 )
                nCol0 = nWidth;
            m_Timetable.SetItemText( uRow, lvi.iSubItem, csFromTime );

            // Add the To time.
            lvi.iSubItem = 1;  // Column
            m_Timetable.InsertItem( &lvi );
            FormatTime( csToTime );
            // Get width of this string.
            nWidth = m_Timetable.GetStringWidth( csToTime );
            if ( nWidth > nCol1 )
                nCol1 = nWidth;
            m_Timetable.SetItemText( uRow, lvi.iSubItem, csToTime );

            // Add the annotation string.
            if ( m_bUse3Cols )
            {
                lvi.iSubItem = 2;  // Column
                m_Timetable.InsertItem( &lvi );

                // Valid the abbreviation number.
                INT nAbbrevIndex = _ttoi( sa->GetAt( sa->GetUpperBound() ) );
                // Use the blank abbreviation if the number is invalid.
                if ( ( nAbbrevIndex < 0 ) || ( nAbbrevIndex > m_saAbbreviations.GetUpperBound() ) ) 
                    nAbbrevIndex = BLANK_ABBREV;
                csAbbrev = m_saAbbreviations.GetAt( nAbbrevIndex );
                // Get width of this string.
                nWidth = m_Timetable.GetStringWidth( csAbbrev );
                if ( nWidth > nCol2 )
                    nCol2 = nWidth;
                m_Timetable.SetItemText( uRow, lvi.iSubItem, csAbbrev );
            }
            uRow++;
        }
    }

    // Change the column widths, with padding so data not abbreviated.
    m_Timetable.SetColumnWidth( 0, nCol0 + 20 );
    m_Timetable.SetColumnWidth( 1, nCol1 + 20 );
    m_Timetable.SetColumnWidth( 2, nCol2 + 20 );
}

BOOL CTrainSchedDlg::SetComboBoxSelection()
{
    // Set combo boxes to stored values if valid.
    if ( ( m_uDay < 0 ) || ( m_uDay >= (UINT) m_DayCombo.GetCount() ) )
        m_uDay = 0;
    m_DayCombo.SetCurSel( m_uDay );
    if ( ( m_uFromStation < 0 ) || ( m_uFromStation >= (UINT) m_FromCombo.GetCount() ) )
        m_uFromStation = 0;
    m_FromCombo.SetCurSel( m_uFromStation );
    if ( ( m_uToStation < 0 ) || ( m_uToStation >= (UINT) m_ToCombo.GetCount() ) )
        m_uToStation = 0;
    m_ToCombo.SetCurSel( m_uToStation );

    return TRUE;
}

void CTrainSchedDlg::ClearUpTimetableData()
{
    // Only clean up if it is populated.
    // Otherwise get error with invalid GetUpperBound() result.
    INT i;
    if ( m_paTimes.GetData() != NULL )
    {
        // Clean up the CPtrArray variables.
		for ( i = 0; i <= m_paTimes.GetUpperBound(); i++ )
		{
            CPtrArray *pa = (CPtrArray *) m_paTimes.GetAt( i );
			for ( INT j = 0; j <= pa->GetUpperBound(); j++ )
			{
                CStringArray *sa = (CStringArray *) pa->GetAt( j );
				sa->RemoveAll();
				delete sa;
            }
			pa->RemoveAll();
			delete pa;
        }
    }
    m_paTimes.RemoveAll();

    m_uaIndexArray.RemoveAll();

    // Empty the combo boxes.
    INT nComboCount = m_DayCombo.GetCount();
    for ( i = 0; i <= nComboCount; i++ )
        m_DayCombo.DeleteString( 0 );
    nComboCount = m_FromCombo.GetCount();
    for ( i = 0; i <= nComboCount; i++ )
    {
        m_FromCombo.DeleteString( 0 );
        m_ToCombo.DeleteString( 0 );
    }
}

BOOL CTrainSchedDlg::ReadTimetableFile()
{

    CFile dbFile;
    CFileException e;
    INT i;

    // Ensure the filename is set.
    if ( m_dbFileName.GetLength() == 0 )
        return FALSE;

    if ( ! dbFile.Open( m_dbFileName, CFile::modeRead | CFile::shareDenyWrite, &e ) )
    {
        CString sErrorMsg;
        e.GetErrorMessage( sErrorMsg.GetBuffer( 1024 ), 1024 );
        sErrorMsg.ReleaseBuffer();
        CString sTitle( (LPCSTR) IDS_TRAINSCHED_ERROR );
        CString sMsg( (LPCSTR) IDS_SELECT_ANOTHER_FILE );
        sErrorMsg += _T( "\n" ) + sMsg;
        MessageBox( sErrorMsg, sTitle );

        OnSelectTimetable();

        // OnSelectTimetable() reads and reloads timetable.
        /*if ( ReadTimetableFile() )
            ReloadTimetable();*/

        return FALSE;
    }

    if ( CFile::hFileNull == dbFile.m_hFile )
    {
        CString sTitle( (LPCSTR) IDS_TRAINSCHED_ERROR );
        CString sErrorMsg( (LPCSTR) IDS_ERROR_OPEN_FILE );
        MessageBox( sErrorMsg, sTitle );

        return FALSE;
    }
    else
    {
        ClearUpTimetableData();

        DWORD dwDbFileLength = dbFile.GetLength();
        LPTSTR lpdbFileData = (LPTSTR) new TCHAR [ dwDbFileLength + 1 ];
        lpdbFileData[ dwDbFileLength ] = NULL;
        dbFile.Read( lpdbFileData, dwDbFileLength );
        dbFile.Close();

        CPtrArray paLines;
		LPTSTR lpNull;
		LPTSTR lpCurrent = lpdbFileData;

#ifdef _UNICODE
        // Unicode text files start with 0xFFEE so move over that byte.
        if ( *lpCurrent == 0xFEFF )
            lpCurrent++;
        else
        {
            // Convert non-Unicode data to Unicode.
            LPWSTR lpwdbFileData = NULL;
            // Find out how many chars needed when data converted to wide chars.
            INT nBuffSize = MultiByteToWideChar( CP_ACP, MB_COMPOSITE, (LPCSTR)lpdbFileData, dwDbFileLength, lpwdbFileData, 0 );
            // Allocate necessary buffer.
            lpwdbFileData = (LPWSTR) new TCHAR [ nBuffSize + 1 ];
            // Do conversion of ANSI data to wide chars.
            MultiByteToWideChar( CP_ACP, MB_COMPOSITE, (LPCSTR)lpdbFileData, dwDbFileLength, lpwdbFileData, nBuffSize );
            // Free current buffer and point current buffer to new buffer.
            ::delete( lpdbFileData );
            lpdbFileData = lpwdbFileData;
            lpCurrent = lpdbFileData;
        }
#endif

        LPTSTR lpStrStart = lpCurrent;
        while ( lpCurrent < ( lpdbFileData + dwDbFileLength ) )
		{
            // Look for 0x0A and 0x0D for Windows and Unix line end chars.
			while ( ( *lpCurrent ) && ( *lpCurrent != 0x0A ) && ( *lpCurrent != 0x0D ) )
			    lpCurrent++;
			lpNull = lpCurrent;
			lpCurrent++;
            // Move past the second line end char if present.
			if ( ( *lpCurrent == 0x0D ) || ( *lpCurrent == 0x0A ) )
				    lpCurrent++;
			*lpNull = NULL;
            paLines.Add( lpStrStart );
            lpStrStart = lpCurrent;
		}

		CString csSplit;
		// Parse out the stations.
		{
            INT nFind = 0;
            CString strOriginal/*( (LPCTSTR) paLines.GetAt( 0 ) )*/, strFound;
            strOriginal = (LPCTSTR) paLines.GetAt( 0 );

            while ( nFind != -1 )
            {
                nFind = strOriginal.FindOneOf( _T( "\t" ) );
                if ( ( nFind != -1 ) || ( strOriginal.GetLength() ) )
                {
                    if ( nFind == -1 )
                        nFind = strOriginal.GetLength();

                    strFound = strOriginal.Mid( 0, nFind );
                    strOriginal = strOriginal.Mid( nFind + 1 );

                    // Don't add "DUMMY" to the combo boxes.
                    if ( strFound.Compare( _T( "DUMMY" ) ) == 0 )
                        m_bUse3Cols = TRUE;
                    else
                    {
                        m_ToCombo.AddString( strFound );
                        m_FromCombo.AddString( strFound );
                    }
                }
            }

            // Validate m_FromStation and m_ToStation and set combo box current selection.
            if ( ( m_uFromStation < 0 ) || ( m_uFromStation >= (UINT) m_FromCombo.GetCount() ) )
                m_uFromStation = 0;
            m_FromCombo.SetCurSel( m_uFromStation );
            m_FromCombo.EnableWindow();
            if ( ( m_uToStation < 0 ) || ( m_uToStation >= (UINT) m_ToCombo.GetCount() ) )
                m_uToStation = 0;
			m_ToCombo.SetCurSel( m_uToStation );
            m_ToCombo.EnableWindow();
        }

        // Enable the Abbreviations menu item to explain 3rd column.
/* TODO        CMenu* pSysMenu = GetSystemMenu(FALSE);
        if (pSysMenu != NULL)
            if ( m_bUse3Cols )
                pSysMenu->EnableMenuItem( IDM_ABBREVIATIONS, MF_BYCOMMAND | MF_ENABLED );
            else
                pSysMenu->EnableMenuItem( IDM_ABBREVIATIONS, MF_BYCOMMAND | MF_DISABLED | MF_GRAYED );
*/

        // Parse out the days.
		{
            INT nFind = 0;
            CString strOriginal( (LPCTSTR) paLines.GetAt( 1 ) ), strFound;
            while ( nFind != -1 )
            {
                nFind = strOriginal.FindOneOf( _T( "\t" ) );
                if ( ( nFind != -1 ) || ( strOriginal.GetLength() ) )
                {
                    if ( nFind == -1 )
                        nFind = strOriginal.GetLength();

                    strFound = strOriginal.Mid( 0, nFind );
                    strOriginal = strOriginal.Mid( nFind + 1 );
                    m_DayCombo.AddString( strFound );
                }
            }
            // Validate m_uDay and set combo box current selection.
            if ( ( m_uDay < 0 ) || ( m_uDay >= (UINT) m_DayCombo.GetCount() ) )
                m_uDay = 0;
			m_DayCombo.SetCurSel( m_uDay );
            m_DayCombo.EnableWindow();
		}

        // Enable the Reverse button.
        GetDlgItem( IDC_REVERSE_BUTTON )->EnableWindow();

        // Parse out index numbers.
		{
            INT nFind = 0;
            CString strOriginal( (LPCTSTR) paLines.GetAt( 2 ) ), strFound;
            while ( nFind != -1 )
            {
                nFind = strOriginal.FindOneOf( _T( "\t" ) );
                if ( ( nFind != -1 ) || ( strOriginal.GetLength() ) )
                {
                    if ( nFind == -1 )
                        nFind = strOriginal.GetLength();

                    strFound = strOriginal.Mid( 0, nFind );
                    strOriginal = strOriginal.Mid( nFind + 1 );

                    m_uaIndexArray.Add( _ttoi( strFound ) );
                }
            }
		}

        // There must be twice as many indices as days (two directions - outbound and return).
        if ( ( m_uaIndexArray.GetUpperBound() + 1 ) != ( 2 * m_DayCombo.GetCount() ) )
        {
            // TODO: Put this in a string table.
            AfxMessageBox( _T( "Problem: Invalid number of indices." ), MB_OK );
            return FALSE;
        }


        // Parse out times.
		for ( i = 0; i <= m_uaIndexArray.GetUpperBound(); i++ )
		{
			UINT uEndLine;
			if ( i == m_uaIndexArray.GetUpperBound() )
				uEndLine = paLines.GetUpperBound() + 1;
			else
				uEndLine = m_uaIndexArray.GetAt( i + 1 ) - 1;

			CPtrArray *p_paOneDir = new CPtrArray;
			m_paTimes.Add( p_paOneDir );

            // Parse out lines grouped by the index numbers.
			for ( UINT uLine = m_uaIndexArray[ i ] - 1; uLine < uEndLine; uLine++ )
			{
				CStringArray *p_saOneTrain = new CStringArray;

                INT nFind = 0;
                CString strOriginal, strFound;
                strOriginal = (LPCTSTR) paLines.GetAt( uLine );
                while ( nFind != -1 )
                {
                    nFind = strOriginal.FindOneOf( _T( "\t" ) );
                    if ( ( nFind != -1 ) || ( strOriginal.GetLength() ) )
                    {
                        if ( nFind == -1 )
                            nFind = strOriginal.GetLength();

                        strFound = strOriginal.Mid( 0, nFind );
                        strOriginal = strOriginal.Mid( nFind + 1 );

                        p_saOneTrain->Add( strFound );
                    }
                }

                // Ensure number of items on the line are valid.
                UINT uExpectedTimes = m_FromCombo.GetCount() - 1;
                // If 3 cols, the extra 'time' will be the annotation item.
                if ( m_bUse3Cols )
                    uExpectedTimes++;
                // Only add p_saOneTrain if the expected number of times are are on this line.
                if ( uExpectedTimes == (UINT) p_saOneTrain->GetUpperBound() )
				    p_paOneDir->Add( p_saOneTrain );
			}
		}

        // Remove NULLs so memory can be freed.
		memset( lpdbFileData, 'x', dwDbFileLength );
		::delete( lpdbFileData );

        // Initialise the table. Only needs to be done once.
        if ( ! m_bTableInitialized )
        {
    	    UINT col;
    	    LV_COLUMN lvc;
            lvc.fmt = LVCFMT_LEFT;
            lvc.mask = LVCF_WIDTH | LVCF_FMT;
            // Split table into 3 even columns.
            RECT rect;
            m_Timetable.GetClientRect( &rect );
            UINT uTableColumns = ( m_bUse3Cols ? 3 : 2 );
            lvc.cx = rect.right / uTableColumns;
//lvc.cx = 40;

            // Add the columns. One column implied so add cols 2 and 3.
            for ( col = 1; col <= uTableColumns; col++ )
            {
                lvc.iSubItem = col;
                m_Timetable.InsertColumn( 0, &lvc );
            }
            m_bTableInitialized = TRUE;
        }
    }

    return TRUE;
}

void CTrainSchedDlg::OnLButtonDown(UINT nFlags, CPoint point) 
{
    CMenu menu;
    VERIFY(menu.LoadMenu(IDR_MENU));
    CMenu* pPopup = menu.GetSubMenu(0);
    ASSERT(pPopup != NULL);
    pPopup->TrackPopupMenu(TPM_LEFTALIGN, point.x, point.y, AfxGetMainWnd());   
	
	CDialog::OnLButtonDown(nFlags, point);
}

void CTrainSchedDlg::OnAbbreviations() 
{
    MessageBox( csAbbrevMessage );
}

void CTrainSchedDlg::OnAboutbox() 
{
    CAboutDlg dlgAbout;
    dlgAbout.DoModal();
}

void CTrainSchedDlg::OnSelectTimetable() 
{
    CString csTypes, csSpec;
    csTypes.LoadString( IDS_FILE_TYPES );
    csSpec.LoadString( IDS_FILE_TYPE_SPEC );

    CFileDialog dlgDBFile( TRUE, csSpec, NULL, OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST | OFN_HIDEREADONLY, csTypes, this );
    if ( dlgDBFile.DoModal() == IDOK )
    {
        m_dbFileName = dlgDBFile.GetPathName();

        if ( ReadTimetableFile() )
        {
            // Display first route and default direction as new timetable loaded.
            m_uDay = 0;
            m_uFromStation = 0;
            m_uToStation = 0;
            SetComboBoxSelection();
            ReloadTimetable();
        }
    }
}

void CTrainSchedDlg::OnOK() 
{
    ClearUpTimetableData();

    // Store preferences.
    CWinApp* pApp = AfxGetApp();
    pApp->WriteProfileString( INI_SECTION, INI_DBFILENAME, m_dbFileName );
    CString csPrefStr;
    csPrefStr.Format( _T( "%u" ), m_uDay );
    pApp->WriteProfileString( INI_SECTION, INI_DAY, csPrefStr );
    csPrefStr.Format( _T( "%u" ), m_uFromStation );
    pApp->WriteProfileString( INI_SECTION, INI_FROM, csPrefStr );
    csPrefStr.Format( _T( "%u" ), m_uToStation );
    pApp->WriteProfileString( INI_SECTION, INI_TO, csPrefStr );

    // Store abbreviations.
    CString csKey;
    for ( INT i = 1; i <= m_saAbbreviations.GetUpperBound(); i++ )
    {
        csKey.Format( _T( "%s%d" ), INI_ABBREV_BASE, i );
        pApp->WriteProfileString( INI_ABBREV_SECTION, csKey, m_saAbbreviations.GetAt( i ) );
    }

    CDialog::OnOK();
}

void CTrainSchedDlg::FormatTime(CString &csTime)
{
    INT nColon = csTime.Find( _T( ':' ) );
    if ( nColon != -1 )
    {
        SYSTEMTIME stTime;
        // Populate struct with valid values in case GetTimeFormat() uses them. */
        GetLocalTime( &stTime );

        stTime.wHour = (unsigned short) _ttoi( csTime.Left( nColon ) );
        stTime.wMinute = (unsigned short) _ttoi( csTime.Mid( nColon + 1 ) );
        /* Format time, without seconds, in user's chosen locale format. */
        GetTimeFormat( LOCALE_USER_DEFAULT, TIME_NOSECONDS, &stTime, NULL, csTime.GetBuffer( MAX_PATH ), MAX_PATH );
        csTime.ReleaseBuffer();
    }
}

void CTrainSchedDlg::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
        // Load timetable as dialog is first painted. This allows error messages
        // to appear on top of the dialog instead of alone, without any dialog.
        if ( m_bFirstPaint )
        {
            m_bFirstPaint  = FALSE;
            if ( m_dbFileName.GetLength() == 0 )
            {
                // Prompt the user to choose a data file, quitting if they don't.
                CString sMsg;
                sMsg.LoadString( IDS_NO_FILE_PREF );
                MessageBox( sMsg );
                OnSelectTimetable();

                if ( m_dbFileName.GetLength() == 0 )
                {
                    sMsg.LoadString( IDS_NO_DATA_FILE );
                    MessageBox( sMsg );
                    OnOK();
                }
            }
        }
	
	// Do not call CDialog::OnPaint() for painting messages
}
